import engine, graphics, gui, thorpy, pygame, reactions, ia

class Game(object):

    def __init__(self, n, nx, ny, d, gap, p1name="Player 1", p2name="Player 2",
                    color_p1=(0,255,0), color_p2=(0,0,255), bckgr=(255,255,255)):
        self.board = engine.Board(n, nx, ny)
        self.coin1 = graphics.get_round(int(0.9*d)+2, 0.2, color_p1)
        self.coin2 = graphics.get_round(int(0.9*d)+2, 0.2, color_p2)
        self.e_c = gui.get_e_coin(d, color_p1)
        self.gap = gap
        self.e_g = gui.get_e_grid(self.board, d, (150,150,150), bckgr, gap)
        self.e_quit = thorpy.make_button("Quit game", thorpy.functions.quit_menu_func)
        self.e_cancel = thorpy.make_button("Cancel last move", self.cancel_move_graphical)
        self.e_box = thorpy.Box.make([self.e_quit, self.e_cancel])
        self.e_text = thorpy.make_text(p1name+" plays")
        self.p1, self.p2 = p1name, p2name
        self.e_bck = thorpy.Background(color=bckgr, elements=[self.e_text,
                                                self.e_c, self.e_g, self.e_box])
        self.e_bck.finish()
        self.e_box.stick_to("screen", "left", "left")
        self.e_text.stick_to("screen", "top", "top")
        self.e_text.move((0,30))
        self.grid = thorpy.Grid(nx, ny, (d+gap,d+gap), self.e_g.get_fus_topleft())
        self.d = d
        self.column = 0
        self.highlight_coord = None
##        self.ia = {-1:None, 1:ia.MinMaxSolver(self.board, 5)}
        self.ia = {1:None, -1:None}
##        self.ia = {-1:None, 1:ia.CompositeSolver(self.board, 6, 300)}
##        self.ia = {1:ia.BasicRandomSolver(self.board, 300),
##                    -1:ia.ImprovedRandomSolver(self.board, 30)}
        #
        reactions.set_grid_reactions(self)
        self.refresh_coin(blit=False)
        self.set_column(0, blit=False)
##        self.put_ia()

    def redraw_all(self):
        self.e_bck.unblit_and_reblit()
        history = list(self.board.history)
        self.board.unput(len(self.board.history))
        self.refresh_coin(False)
        for column in history:
            self.board.put(column)
            self.blit_put(column, False)
            self.refresh_coin(False)
        assert history == self.board.history

    def cancel_move_graphical(self):
        if self.board.history:
            print("cancel")
            self.board.unput(1)
            self.redraw_all()
            self.refresh_coin()
            self.highlight_coord = None

    def set_column(self, x, blit=True):
        self.column = x
        if x is not None:
            rect = self.grid.get_rect_at_coord((x, 0))
            y = self.e_g.get_fus_rect().top - 1.25*self.d
            if blit:
                self.e_c.unblit_and_reblit_func(self.e_c.set_topleft,
                                                pos=(rect.x+self.gap//2, y))

    def refresh_coin(self, blit=True):
        img = self.coin1 if self.board.v == 1 else self.coin2
        if blit:
            self.e_c.unblit_and_reblit_func(self.e_c.set_image, img=img)
        else:
            self.e_c.set_image(img)

    def refresh_text(self):
        if self.board.v == 1:
            text = self.p1 + " plays"
        elif self.board.v == -1:
            text = self.p2 + " plays"
        self.e_text.unblit_and_reblit_func(self.e_text.set_text, text=text)


    def toggle_coin(self):
        if self.e_c.get_image() == self.coin1:
            self.e_c.set_image(self.coin2)
        elif self.e_c.get_image() == self.coin2:
            self.e_c.set_image(self.coin1)
        else:
            raise Exception()

    def blit_put(self, column, highlight):
        y = self.board.free_cells[column]-1
        self.draw(column, y, highlight)
        return column, y

    def draw(self, x, y, highlight):
        cr = self.grid.get_rect_at_coord((x,y))#.inflate((-10,-10))
        img = self.e_c.get_image()
        r = img.get_rect()
        r.center = cr.center
        thorpy.functions.get_screen().blit(img, r.topleft)
        if highlight:
            pygame.draw.rect(thorpy.functions.get_screen(), (0,0,0),
                                r.inflate((-r.w//2, -r.w//2)))
        pygame.display.update(r)

    def put(self, column=None):
        column = self.column if column is None else column
        if column is not None:
            if self.board.put(column):
                self.refresh_text()
                if self.highlight_coord:
                    self.toggle_coin()
                    self.draw(self.highlight_coord[0], self.highlight_coord[1],
                                False)
                    self.toggle_coin()
                self.highlight_coord = self.blit_put(column, True)
                self.refresh_coin()
                if self.board.won:
                    print("won", self.board.won)
                    if self.board.won == 1:
                        text = self.p1 + " won the game."
                    else:
                        text = self.p2 + " won the game"
                    self.e_text.unblit();self.e_text.update()
                    self.e_text.set_text(text)
                    self.e_text.set_font_color((255,0,0))
                    self.e_text.set_font_size(thorpy.style.FONT_SIZE+3)
                    self.e_text.unblit_and_reblit()
                    self.e_g.active = False
                    return
                self.put_ia()

    def put_ia(self):
        if len(self.board.history) < len(self.board):
            if self.ia[self.board.v] is not None:
##                print("choose", self.board.v)
                col_ia = self.ia[self.board.v].choose_column()
                self.put(col_ia)


